/**
 * Post-install script to pre-configure the config files.
 */
function postInstall()
{	
	
	log( 2, "Starting postInstall()..." );
	
	try
	{
		// --- Pre-configure the files ---
		var serverPropsFilename;
		var ProvisionXMLFilename;
		var newProvisionXMLFilename;
		var ncsfServerPropsPath;
		
		if (ncipackage.shortArch == "win32" ) {
			serverPropsFilename = getNchome() + "\\guifoundation\\conf\\server.props";
			newServerPropsFilename = getNchome() + "\\etc\\sm\\server.props";
			ProvisionXMLFilename = getNchome() + "\\guifoundation\\webapps\\desktop\\xml\\00_smprovision_init.xml";
			newProvisionXMLFilename = getNchome() + "\\etc\\guifoundation\\antonce\\00_smprovision_init.xml";
			ncsfServerPropsPath = getNchome() + "\\guifoundation";
		} else {
			serverPropsFilename = getNchome() + "/guifoundation/conf/server.props";
			newServerPropsFilename = getNchome() + "/etc/sm/server.props";
			ProvisionXMLFilename = getNchome() + "/guifoundation/webapps/desktop/xml/00_smprovision_init.xml";
			newProvisionXMLFilename = getNchome() + "/etc/guifoundation/antonce/00_smprovision_init.xml";
			ncsfServerPropsPath = getNchome() + "/guifoundation";
		}
		
		// Deal with Server.props
		log( 2, "Pre-configuring " + serverPropsFilename );
		var serverPropsContents = readFromFile( serverPropsFilename );
		backupFile( serverPropsFilename );
		var newserverPropsContents = processServerProps( serverPropsContents );
		writeToFile( newServerPropsFilename, newserverPropsContents );		
		log( 3, serverPropsFilename + " pre-configuration completed." );
				
		// Now deal with the provisionnal system
		/* JMP: This is not supported with HP/UX. Nee
		log( 2, "Pre-configuring " + ProvisionXMLFilename );
		var ProvisionXMLContents = LoadXMLFile( ProvisionXMLFilename );
		UpdateXMLAttribute( ProvisionXMLContents , "//property[@name='server']" , "value" , ncsfServerPropsPath );
		SaveXMLFile( ProvisionXMLContents , newProvisionXMLFilename);		
		log( 3, ProvisionXMLFilename + " pre-configuration completed." );		
		*/

		log( 2, "Pre-configuring " + ProvisionXMLFilename );
		var ProvisionXMLFilenameContent = readFromFile( ProvisionXMLFilename );		
		var newProvisionXMLFilenameContent = UpdateProvisionXMLFile( ProvisionXMLFilenameContent , ncsfServerPropsPath );		
		writeToFile( newProvisionXMLFilename, newProvisionXMLFilenameContent );		
		log( 3, ProvisionXMLFilename + " pre-configuration completed." );		
		
	}	
	
	catch ( e )
	{
		log( 0, e );
	}
	
	log( 3, "postInstall() completed." );
}



/**
 * Process the content of the server.props file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processServerProps( fileContents )
{
	log( 2, "Processing definitions file..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

	// Replace port name (in all places).
	var regex = /8077/g;
	var newContents = fileContents.replace( regex, propMgr.GetProp( "NCSM0.HTTPPORT" ).value );
	
	var regex = /localhost/g;
	var newContents = newContents.replace( regex, propMgr.GetProp( "NCSM0.HOST" ).value );
		
	var password = ncsm_crypt( propMgr.GetProp( "NCSM0.ADMIN.PASSWORD" ).value );

	var regex = /__NCSM0\.ADMIN\.PASSWORD__/g;
	var newContents = newContents.replace( regex, password );
	
	return newContents;
}


function UpdateProvisionXMLFile( fileContents , ncsfServerPropsPath  )
{
	//		UpdateXMLAttribute( ProvisionXMLContents , "//property[@name='server']" , "value" , ncsfServerPropsPath );
	
	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

	// Replace server name (in all places).
	var regex = /\/opt\/netcool\/guifoundation/g;
	var newContents = fileContents.replace( regex , ncsfServerPropsPath  );
	
	// Replace port number (in all places).
	var regex = /8080/g;
        var newContents = newContents.replace( regex, propMgr.GetProp( "GUIFOUNDATION.WEBSERVER.PORT" ).value );

	var regex = /__ADMIN_USERNAME__/g;
	var newContents = newContents.replace( regex, propMgr.GetProp( "NCSM0.ADMIN.USERNAME" ).value );

	var regex = /__ADMIN_PASSWORD__/g;
	var newContents = newContents.replace( regex, propMgr.GetProp( "NCSM0.ADMIN.PASSWORD" ).value );
	
	var regex = /__NCHOME__/g;
	var newContents = newContents.replace( regex, propMgr.GetProp( "NCHOME.INSTALLDIR" ).value );
		
	
	return newContents;
	
}


/************************************************************************************************/
/**                                                                                            **/
/**                                                                                            **/
/**   Utilities Javascript Functions for Installers Pore/post install actions                  **/
/**                                                                                            **/
/**                                                                                            **/
/************************************************************************************************/

/**
 * Read the contents of a file.
 *
 * @param filename (String) The full path of the file to read from.
 * @return (String) Contents of the file.
 */
function readFromFile( filename )
{
	// Read the file.
	log( 2, "Reading file: " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	// NOTE: This will not work with Mac!
	file.initWithPath( filename );

	// Use IOService to convert the file to a file URI.
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService( Components.interfaces.nsIIOService );
	var fileUri = ioService.newFileURI( file.QueryInterface( Components.interfaces.nsIFile ) );

	// Get a channel from the URI.
	var channel = ioService.newChannelFromURI( fileUri );

	// Create an input stream.
	var inStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance( Components.interfaces.nsIScriptableInputStream );

	var fileContents = "";

	try
	{
		inStream.init( channel.open() );

		var bytes;

		while ( (bytes = inStream.available()) > 0 )
		{
			fileContents += inStream.read( bytes );
		}
	}
	catch ( e )
	{
		log( 0, e );
	}
	finally
	{
		inStream.close();
	}
	
	log( 3, "Read completed." );
	
	return fileContents;
}

/**
 * Backup a file by appending ".bak" to the filename.
 *
 * @param filename (String) The full path of the file to backup.
 */
function backupFile( filename )
{
	log( 2, "Backing up " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	file.initWithPath( filename );
	
	file.moveTo( file.parent, file.leafName + ".bak" );
	
	log( 3, "Backup completed." );
}


/**
 * Write content to a file. The file will have 664 permissions.
 *
 * @param filename (String) The full path of the file to read from.
 * @param content (String) The content to write.
 */
function writeToFile( filename, content )
{
	log( 2, "Writing to file: " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	file.initWithPath( filename );

	// Force the creation of the destication folder, just in case.
	try {
		file.parent.create( 1 , 0775 );
	} catch (e) {
	}

	// To write to the file, make an output stream, initialize the stream with
	// the file, and write to it.
	var outStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
	
	try
	{
		outStream.init( file.QueryInterface( Components.interfaces.nsIFile ), 0x04 | 0x08, 0664, 0 );
		// octal 0664 gives "664" permissions.
		var result = outStream.write( content, content.length );
	}
	catch ( e )
	{
		// Log!
		log( 0, e );
	}
	finally
	{
		outStream.close();
	}
	
	log( 3, "Write completed." );
}


// Get $NCHOME.
function getNchome()
{
	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

	return propMgr.GetProp( "NCHOME.INSTALLDIR" ).value;
}


/**
 * Log a message.
 *
 * @param level (int) The level to log with. Valid levels are: 0 - Error,
 *        1 - Warning 2 - Info, 3 - Debug.
 * @param msg (String) The message to log.
 */
function log( level, msg )
{
	try
	{
		var logger = Components.classes["@micromuse.com/NCILog;1"].getService( Components.interfaces.nsINCILog);
		
		switch ( level )
		{
			case 0: logger.error( msg ); break;
			case 1: logger.warning( msg ); break;
			case 2: logger.info( msg ); break;
			case 3: logger.debug( msg ); break;
			default: break;
		}
	}
	catch ( e )
	{
		// ??
	}
}


/** 
 * Load XML File as XML 
 *
 * @param	filename (string) the local XML file to load 
 *
 * @return 	(DOMXMLDocument) The XML Document with its parser
 * 
 */
function LoadXMLFile( filename )
{
	/** Step1: Make a proper URL from the file name. need to have a full path here... */
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	file.initWithPath( filename );


	var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	fileuri = ios.newFileURI( file.QueryInterface( Components.interfaces.nsIFile ));
	var fileurl = fileuri.asciiSpec ;



	/** Step2: Create an XML parser and load the XML into the parser using the url crete in Step1 */
	var xmlparser = Components.classesByID["{a6cf9063-15b3-11d2-932e-00805f8add32}"].createInstance( Components.interfaces.nsIDOMXMLDocument );
	xmlparser.async = false;

	rc = xmlparser.load( fileurl );
	
	return xmlparser;
}

/**	Update a XML Noide attribute. the node is located using a XPath location expression.
 *
 * 	@param	xml 		(XMLDOMDocument)	The XML Document to update
 *  @param	xpathnode	(string)			The XPAth compatible expression to locate the node (i.e. "//page[@step='100']" ).
 *	@param	name		(string)			The name of the attribute to update on the node
 *	@param	value		(string)			The new value for the node.
 */
function UpdateXMLAttribute( xml , xpathnode , name , value )
{
	var xmlSet = xml.QueryInterface( Components.interfaces.nsIDOMXPathEvaluator ).evaluate( xpathnode , xml , null , 5 , null );
	var node = xmlSet.QueryInterface( Components.interfaces.nsIDOMXPathResult).iterateNext();
	if ( node ) {
		var titlenode = node.QueryInterface( Components.interfaces.nsIDOMElement).getAttribute( name );
		node.QueryInterface( Components.interfaces.nsIDOMElement).setAttribute( name , value );
	} else {
		log( 3, "Node " + xpathnode + " not found ");
	}	
}


/** Save XMLDocument to a local file
 *
 *	@param	xml  (XMLDOMDocument)	the XML Document to save,
 *	@param	filename (string)	name of the local file, using a local full qualified path
 *
 */
function SaveXMLFile( xml , filename )
{
	log( 1 , "entering SaveXMLFile( " + filename + " )" );
	
	try {
		/* Create the file */
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

		file.initWithPath( filename );
		
		/** Create the folder just in case */
		try {
			file.parent.create( 1 , 0775 );
		} catch (e) {
		}
		

		/* convert the file into an output stream */
		var of = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream );
		of.init( file.QueryInterface( Components.interfaces.nsIFile ) , 0x04 | 0x08, 0664, 0 );

		/** Serialize the XML document into the file stream created above */
		var xmlserial = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer );

		/* Close the Stream */	
		xmlserial.serializeToStream( xml , of , null );

		of.close();

		log( 3 , "SaveXMLFile Done!" );
	} 
	catch( e )
	{
		log( 0, "Error in SaveXMLFile: " + e );
		throw e;
	}
}


//////////////////////////////////////////////////////////////////
//
// Trim in Javascript...
//
//////////////////////////////////////////////////////////////////

function Trim(strValue){
	return LTrim(RTrim(strValue));
}

function LTrim(strValue) {
	var LTRIMrgExp = /^\s */;
	return strValue.replace(LTRIMrgExp, '');
}

function RTrim(strValue) {
	var RTRIMrgExp = /\s *$/;
	return strValue.replace(RTRIMrgExp, '');
}

//////////////////////////////////////////////////////////////////
//
// Lifted from Function.js
// version 1.2
//
// contributors:
//  Jason Staples (most of it)
//  Shoel Perelman
//  Adrian Fowler (log function from Webtop's ActionUtils)
//
//////////////////////////////////////////////////////////////////

var separator="/";
var extension = "";
var i=0;
var arch = "";
var ncp = ncipackage;

if (ncp != null) {
	arch = ncipackage.shortArch;
}

if (arch == "win32") {
	separator="\\";
  	extension = ".bat";
}

function makeDir(directory) {
	try {
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
		file.initWithPath(directory);
		if (!file.exists()) {
			file.create(1, 0777);
		}
	}
	catch(e) {
		log(0,"ERROR:ncipackage(" + ncipackage.name + "):javascript:SecMngrClient.js:mkdir(" + directory + "):" + e );
		throw(e);
	}
}

function readFile(filename) {
	try {
		var data = "";
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
		file.initWithPath(filename);
		if (file.exists()) {
			var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
			fstream.init(file, 1, 0, false);
			sstream.init(fstream); 
			data += sstream.read(-1);
			sstream.close();
			fstream.close();
		}
		return data;
	}
	catch(e) {
		log(0,"ERROR:ncipackage(" + ncipackage.name + "):javascript:SecMngrClient.js:readFile(" + filename + "):" + e );
		throw(e);
	}
}

function deleteFile(filename, recursive) {
	try {
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
		file.initWithPath(filename);
		if (file.exists()) {
			file.remove(recursive);
		}
	}
	catch(e) {
		log(0,"ERROR:ncipackage(" + ncipackage.name + "):javascript:SecMngrClient.js:deleteFile(" + filename + "):" + e );
		throw(e);
	}
}


function appendFile(sourcefile, appendstring) {
	try {
		log(2, "appendFile(sourcefile: "+sourcefile+", appendstring: "+appendstring+")");

		// file is nsIFile, data is a string
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
		file.initWithPath(sourcefile);

		if (file.exists()) {
			log(3, "appendFile setting permissions for existing file to | 0x7 for sourcefile: "+sourcefile);
			file.permissions = file.permissions | 0777;
		}

		log(3, "appendFile opened sourcefile: "+sourcefile);
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		//log(0, "got output stream");
		// use 0x02 | 0x10 to open file for appending.
		foStream.init(file, 0x04 | 0x08 | 0x10, 0777, 0); // write, create, truncate
		log(3, "appendFile initialized foStream for sourcefile: "+sourcefile);

		log(3, "appendFile setting new permissions to | 0x7 for sourcefile: "+sourcefile);
		file.permissions = file.permissions | 0777;



		//log(0, "file init");
		foStream.write(appendstring, appendstring.length);
		log(3, "appendFile wrote data: "+appendstring);
		foStream.close();
		log(3, "appendFile closed file");
		return true;
	}
	catch(e) {
		log(0,"ERROR:ncipackage(" + ncipackage.name + "):javascript:SecMngrClient.js:appendFile(" + sourcefile + "):" + e );
		throw(e);
	}
}


function execScript(script) {
	try {
		log(3, "running execScript: " + script );
		
		var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService(Components.interfaces.nsINCIProperties);

		prop = propMgr.GetProp("$NCHOME");
		var nchome = prop.value;

		makeDir(nchome + separator + "tmp");

		var scriptname = nchome + separator + "tmp" + separator + "execpro" + i + extension;
		// create temp file
		appendFile(scriptname, script + "\n");

		var outputscript = nchome + separator + "tmp" + separator + "runexecpro" + i + extension;
		if (arch == "win32") {
			appendFile(outputscript, "@echo off\n");
		} else {
			appendFile(outputscript, "#!/bin/sh\n");
		}
		appendFile(outputscript, "\"" + scriptname + "\" >> \"" + outputscript + ".out\" 2>\"" + outputscript + ".err\"\n");

		// create an nsILocalFile for the executable
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(outputscript);
		//file.initWithPath("c:\\windows\\system32\\cmd.exe");

		// create an nsIProcess
		var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
		process.init(file);

		// Run the process.
		// If first param is true, calling process will be blocked until
		// called process terminates. 
		// Second and third params are used to pass command-line arguments
		// to the process.
		var args = [];
		var outvalue = process.run(true, args, args.length);

		var output = new Object();
		output.stdout = readFile(outputscript + ".out");
		output.stderr = readFile(outputscript + ".err");
		deleteFile(scriptname, true);
		deleteFile(outputscript, true);
		deleteFile(outputscript + ".out", true);
		deleteFile(outputscript + ".err", true);
		i++;

		log(3, "execScript returned stdout: " + output.stdout);
		log(3, "execScript returned stderr: " + output.stderr);
		
		return output;
	}
	catch(e) {
		log(0,"ERROR:ncipackage(" + ncipackage.name + "):javascript:SecMngrClient.js:execScript(" + script + "):" + e );
		throw(e);
	}
}


function ncsm_crypt( password ) {

	log(3, "Entering ncsm_crypt(" + password +")" );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );
	
	var cmd = propMgr.GetProp( "$NCHOME" ).value ;	
	if (arch == "win32") {
		cmd += "\\security\\bin\\ncsm_crypt.bat" ;
		cmd = "\"" + cmd + "\"" ;
	} else {
		cmd += "/security/bin/ncsm_crypt" ;
	}

	cmd += " " + password ;	
	
	var res = execScript( "\nunset NCSM_HOME\n" + cmd );
	var rc = Trim( res.stdout );

	log(3, "ncsm_crypt(" + password +") returns: " + rc );
	
	return rc;
}